class APITokenAuthentication(object):

    @staticmethod
    def apply(http_request):
        """
        Add token authentication to the header request.
        Args:
            http_request (HttpRequest): The HttpRequest object to which authentication will be added.
        """
        pass
